@@ -6,6 +6,13 @@ import json
 import decimal
 
 
+def json_encoder(obj):
+    if isinstance(obj, date):
+        return str(obj.isoformat())
+    if isinstance(obj, decimal.Decimal):
+        return str(obj)
+
+
 class Client(object):
 
     def __init__(self, api_credentials):
@@ -17,6 +24,8 @@ class Client(object):
         return factory.SignedRequestFactory
 
     def _get_response(self, http_method, endpoint, data=None, files=None, **request_kwargs):
+        if request_kwargs.get("headers", {}).get("Content-Type") == "application/json":
+            data = json.dumps(data, default=json_encoder)
         try:
             http_response = urllib2.urlopen(self._get_request(http_method, endpoint, data, files, **request_kwargs))
         except urllib2.HTTPError as e:
