@@ -1,20 +1,13 @@
-from datetime import date
-import decimal
-import json
+import re
+import constants
 import itertools
 import mimetools
 import mimetypes
-import constants
-import request
+from urllib import urlencode, quote
+
 import apysigner
-from urllib import urlencode
 
-
-def json_encoder(obj):
-    if isinstance(obj, date):
-        return str(obj.isoformat())
-    if isinstance(obj, decimal.Decimal):
-        return str(obj)
+from . import request
 
 
 def default_encoding(raw_data, *args):
@@ -22,7 +15,7 @@ def default_encoding(raw_data, *args):
 
 
 def json_encoding(raw_data, *args):
-    return json.dumps(raw_data, default=json_encoder)
+    return raw_data
 
 
 class SignedRequestFactory(object):
@@ -54,16 +47,20 @@ class SignedRequestFactory(object):
         return self._build_signed_url(url, headers)
 
     def _build_signed_url(self, url, headers):
-        data = {} if self.should_data_be_sent_on_querystring() else self._build_signature_friendly_dict_for_content_type(headers)
+        data = {} if self.should_data_be_sent_on_querystring() else self._build_signature_dict_for_content_type(headers)
         signature = apysigner.get_signature(self.private_key, url, data)
-        signed_url = url + "&{}={}".format(constants.SIGNATURE_PARAM_NAME, signature)
+        signed_url = self._escape_url(url) + "&{}={}".format(constants.SIGNATURE_PARAM_NAME, signature)
         return signed_url
 
-    def _build_signature_friendly_dict_for_content_type(self, headers):
+    def _escape_url(self, url):
+        match = re.search(r'(^.+://)([^?]+)(\?.+$)?', url)
+        return match.group(1) + quote(match.group(2)) + (match.group(3) if match.group(3) is not None else '')
+
+    def _build_signature_dict_for_content_type(self, headers):
         content_type = headers.get("Content-Type")
         if content_type and content_type == "application/json":
             encoding_func = self.content_type_encodings.get(content_type, default_encoding)
-            return json.loads(encoding_func(self.raw_data))
+            return encoding_func(self.raw_data)
         return self.raw_data
 
     def _get_data_payload(self, request_headers):
